//Template.C
//Ernest Pazera
//24JUL2001

//super include for cybiko sdk
#include "cywin.h"
#include "StdDlg.c"
#include "MenuForm.c"
#include "Cursor.c"
#include "View.c"
#include "Map.c"
#include "FileListForm.c"

//constants
//time out for checking messages
#define MESSAGE_TIMEOUT 0
//minimum message to check for
#define MESSAGE_MIN 1
//maximum message to check for
#define MESSAGE_MAX MSG_USER

static char* MainMenu[]=
{
	"New Level",
	"Open Level",
	"Save Level",
	"Quit",
	""
};

char* NewMenu[5];

//globals
//main module
struct module_t main_module;
//exit application flag
bool exit_app;
//tileset
struct BitmapSequence bseqTileSet;
VIEW mapview;
struct Bitmap bmpScreen;
struct Font fnt8x8;

struct cCustomForm form;
struct rect_t rect;

//map
MAP map;
CURSOR anchor;
CURSOR cursor;

int Piece;

//functions
//initialize the program
bool Prog_Init();
//clean up the program
void Prog_Done();
//message handler
void Prog_proc(struct Message* ptr_msg);
//quit function
void Quit();

void LoadGfx();
void UnloadGfx();

void Map_show();
void Map_draw();
void Map_draw_square(int x,int y,int mapx,int mapy);

void Cursor_show();
void Cursor_hide();

void NewLevel();
void OpenLevel();
void SaveLevel();

int NewLevelMenu(int w,int h);
int InputBoxNum(char* title,char* prompt,int num);

bool OnKeyDown(int scancode,bool shift,bool ctrl,bool repeat,char ch);
bool OnKeyUp(int scancode,bool shift,bool ctrl,bool repeat,char ch);
bool OnCharTyped(int scancode,bool shift,bool ctrl,bool repeat,char ch);
bool OnTimer();
bool OnPowerDown();
bool OnQuit();
bool OnPaint();
bool OnLostFocus();
bool OnGotFocus();
bool OnFiles();
bool OnLaunch();
bool OnDevice();
bool OnPing();
bool OnShutUp();
bool OnUser(struct Message* ptr_msg);

long main( int argc, char* argv[], bool start )
{
	//message pointer
	struct Message* ptr_msg;

	//initialize module
	init_module(&main_module);

	//initialize
	if(!Prog_Init())
	{
		//return error
		return(1);
	}

	//set exit flag to false
	exit_app=FALSE;
	//main loop
	while(!exit_app)
	{
		//grab a message
		ptr_msg=cWinApp_get_message(main_module.m_process,MESSAGE_TIMEOUT,MESSAGE_MIN,MESSAGE_MAX);
		//send message to the proc
		Prog_proc(ptr_msg);
		//delete message
		Message_delete(ptr_msg);
	}

	//clean up
	Prog_Done();

	//done
	return(0);
}

//initialize the program
bool Prog_Init()
{
	LoadGfx();
	Map_ctor(&map,12,12);

	Cursor_ctor(&cursor,0,0,Map_get_w(&map),Map_get_h(&map),TRUE,TRUE);
	Cursor_ctor(&anchor,0,0,Map_get_w(&map)-11,Map_get_h(&map)-11,FALSE,FALSE);

	Piece=8;
	Map_draw();
	Map_show();
	//return TRUE if initialization is successful
	return(TRUE);
}

//clean up the program
void Prog_Done()
{
	//clean up code goes here
	UnloadGfx();
	Map_dtor(&map,LEAVE_MEMORY);
}

//message handler
void Prog_proc(struct Message* ptr_msg)
{
	//check for a message
	if(ptr_msg)
	{
		switch(ptr_msg->msgid)
		{
		case MSG_KEYDOWN:
			{
				if(!OnKeyDown(Message_get_key_param(ptr_msg)->scancode,(Message_get_key_param(ptr_msg)->mask&KEYMASK_SHIFT)!=0,(Message_get_key_param(ptr_msg)->mask&KEYMASK_CTRL)!=0,(Message_get_key_param(ptr_msg)->mask&KEYMASK_AUTOREPEAT)!=0,Message_get_key_param(ptr_msg)->ch))
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_KEYUP:
			{
				if(!OnKeyUp(Message_get_key_param(ptr_msg)->scancode,(Message_get_key_param(ptr_msg)->mask&KEYMASK_SHIFT)!=0,(Message_get_key_param(ptr_msg)->mask&KEYMASK_CTRL)!=0,(Message_get_key_param(ptr_msg)->mask&KEYMASK_AUTOREPEAT)!=0,Message_get_key_param(ptr_msg)->ch))
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_CHARTYPED:
			{
				if(!OnCharTyped(Message_get_key_param(ptr_msg)->scancode,(Message_get_key_param(ptr_msg)->mask&KEYMASK_SHIFT)!=0,(Message_get_key_param(ptr_msg)->mask&KEYMASK_CTRL)!=0,(Message_get_key_param(ptr_msg)->mask&KEYMASK_AUTOREPEAT)!=0,Message_get_key_param(ptr_msg)->ch))
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_TIMER:
			{
				if(!OnTimer())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_POWERDOWN:
			{
				if(!OnPowerDown())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_QUIT:
			{
				if(!OnQuit())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_PAINT:
			{
				if(!OnPaint())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_FILES:
			{
				if(!OnFiles())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_LOSTFOCUS:
			{
				if(!OnLostFocus())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_GOTFOCUS:
			{
				if(!OnGotFocus())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_LAUNCH:
			{
				if(!OnLaunch())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_DEVICE:
			{
				if(!OnDevice())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_PING:
			{
				if(!OnPing())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		case MSG_SHUTUP:
			{
				if(!OnShutUp())
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		default:
			{
				if(!OnUser(ptr_msg))
				{
					cWinApp_defproc(main_module.m_process,ptr_msg);
				}
			}break;
		}
	}
}

bool OnKeyDown(int scancode,bool shift,bool ctrl,bool repeat,char ch)
{
	int mr;
	//escape key
	if(scancode==KEY_ESC)
	{
		mr=SimpleMenuForm("Main Menu",MainMenu,0,0,160,100,FALSE,main_module.m_process);
		
		switch(mr)
		{
		case 0://new
			{
				NewLevel();
			}break;
		case 1://open
			{
				OpenLevel();
			}break;
		case 2://save
			{
				SaveLevel();
			}break;
		case -2:
		case 3://quit
			{
				//send a quit message
				send_msg(cWinApp_get_name(main_module.m_process),MSG_QUIT,0,0,0,0);
			}break;
		}
		Map_draw();
		Map_show();
		//handled message
		return(TRUE);
	}
	if(scancode==KEY_UP)
	{
		if(ctrl)
		{
			Piece+=12;
			Piece%=16;
			Map_show();
			return(TRUE);
		}
		if(shift)
		{
			Cursor_move_rel(&anchor,0,-1);
			Map_draw();
			Map_show();
			return(TRUE);
		}
		Cursor_move_rel(&cursor,0,-1);
		Map_show();
		return(TRUE);
	}
	if(scancode==KEY_DOWN)
	{
		if(ctrl)
		{
			Piece+=4;
			Piece%=16;
			Map_show();
			return(TRUE);
		}
		if(shift)
		{
			Cursor_move_rel(&anchor,0,1);
			Map_draw();
			Map_show();
			return(TRUE);
		}
		Cursor_move_rel(&cursor,0,1);
		Map_show();
		return(TRUE);
	}
	if(scancode==KEY_LEFT)
	{
		if(ctrl)
		{
			Piece+=15;
			Piece%=16;
			Map_show();
			return(TRUE);
		}
		if(shift)
		{
			Cursor_move_rel(&anchor,-1,0);
			Map_show();
			Map_draw();
			return(TRUE);
		}
		Cursor_move_rel(&cursor,-1,0);
		Map_show();
		return(TRUE);
	}
	if(scancode==KEY_RIGHT)
	{
		if(ctrl)
		{
			Piece+=1;
			Piece%=16;
			Map_show();
			return(TRUE);
		}
		if(shift)
		{
			Cursor_move_rel(&anchor,1,0);
			Map_draw();
			Map_show();
			return(TRUE);
		}
		Cursor_move_rel(&cursor,1,0);
		Map_show();
		return(TRUE);
	}
	if(scancode==KEY_ENTER)
	{
		Map_set_square(&map,Cursor_get_x(&cursor),Cursor_get_y(&cursor),(char)Piece);
		Map_draw();
		Map_show();
		return(TRUE);
	}
	//return false
	return(FALSE);
}

bool OnKeyUp(int scancode,bool shift,bool ctrl,bool repeat,char ch)
{
	//return false
	return(FALSE);
}

bool OnCharTyped(int scancode,bool shift,bool ctrl,bool repeat,char ch)
{
	//return false
	return(FALSE);
}

bool OnTimer()
{
	//return false
	return(FALSE);
}

bool OnPowerDown()
{
	//return false
	return(FALSE);
}

bool OnQuit()
{
	//quit the app
	Quit();
	//return false
	return(TRUE);
}

bool OnPaint()
{
	//return false
	return(FALSE);
}

bool OnLostFocus()
{
	//return false
	return(FALSE);
}

bool OnGotFocus()
{
	Map_draw();
	Map_show();
	//return false
	return(FALSE);
}

bool OnFiles()
{
	//return false
	return(FALSE);
}

bool OnLaunch()
{
	//return false
	return(FALSE);
}

bool OnDevice()
{
	//return false
	return(FALSE);
}

bool OnPing()
{
	//return false
	return(FALSE);
}

bool OnShutUp()
{
	//return false
	return(FALSE);
}

bool OnUser(struct Message* ptr_msg)
{
	//return false
	return(FALSE);
}

//quit function
void Quit()
{
	//set the exit app flag
	exit_app=TRUE;
}

void LoadGfx()
{
	BitmapSequence_ctor_Ex(&bseqTileSet,"tileset.pic");
	View_ctor(&mapview,main_module.m_gfx,2,2,96,96,DM_PUT,CLR_WHITE,BM_NORMAL);
	Bitmap_ctor_Ex1(&bmpScreen,"screen.pic");
	Font_ctor_Ex(&fnt8x8,"DosRomFont8x8.fnt",TRUE,8);
	DisplayGraphics_set_font(main_module.m_gfx,&fnt8x8);
}

void UnloadGfx()
{
	BitmapSequence_dtor(&bseqTileSet,LEAVE_MEMORY);
	View_dtor(&mapview,LEAVE_MEMORY);
	Bitmap_dtor(&bmpScreen,LEAVE_MEMORY);
	DisplayGraphics_set_font(main_module.m_gfx,cool_normal_font);
	Font_dtor(&fnt8x8,LEAVE_MEMORY);
}

void Map_draw()
{
	int x;
	int y;
	for(x=0;x<12;x++)
	{
		for(y=0;y<12;y++)
		{
			Map_draw_square(x,y,x+Cursor_get_x(&anchor),y+Cursor_get_y(&anchor));
		}
	}
}

void Map_show()
{
	char buf[10];
	int x;
	int y;

	DisplayGraphics_set_draw_mode(main_module.m_gfx,DM_PUT);
	DisplayGraphics_draw_bitmap(main_module.m_gfx,&bmpScreen,0,0,BM_NORMAL);

	DisplayGraphics_set_font(main_module.m_gfx,&fnt8x8);
	DisplayGraphics_set_color(main_module.m_gfx,CLR_BLACK);
	sprintf(buf,"%d",Cursor_get_x(&cursor));
	DisplayGraphics_draw_text(main_module.m_gfx,buf,120,4);
	sprintf(buf,"%d",Cursor_get_y(&cursor));
	DisplayGraphics_draw_text(main_module.m_gfx,buf,120,12);
	sprintf(buf,"%d",Map_get_w(&map));
	DisplayGraphics_draw_text(main_module.m_gfx,buf,120,20);
	sprintf(buf,"%d",Map_get_h(&map));
	DisplayGraphics_draw_text(main_module.m_gfx,buf,120,28);
	
	x=Piece%4;
	y=Piece/4;

	DisplayGraphics_draw_rect(main_module.m_gfx,x*12+104,y*12+40,10,10);
	
	Cursor_show();
	View_show(&mapview);
	Cursor_hide();
	DisplayGraphics_show(main_module.m_gfx);
}

void Map_draw_square(int x,int y,int mapx,int mapy)
{
	if(mapx<0 || mapx>=Map_get_w(&map) || mapy<0 || mapy>=Map_get_h(&map))
	{
		View_set_draw_mode(&mapview,DM_PUT);
		View_draw_bitmap(&mapview,BitmapSequence_get_bitmap(&bseqTileSet,17),x*8,y*8,BM_NORMAL);
	}
	else
	{
		View_set_draw_mode(&mapview,DM_PUT);
		View_draw_bitmap(&mapview,BitmapSequence_get_bitmap(&bseqTileSet,Map_get_square(&map,mapx,mapy)),x*8,y*8,BM_NORMAL);
	}
}

void Cursor_show()
{
	View_set_draw_mode(&mapview,DM_XOR);
	View_draw_bitmap(&mapview,BitmapSequence_get_bitmap(&bseqTileSet,16),Cursor_get_x(&cursor)*8-Cursor_get_x(&anchor)*8,Cursor_get_y(&cursor)*8-Cursor_get_y(&anchor)*8,BM_NORMAL);
}

void Cursor_hide()
{
	View_set_draw_mode(&mapview,DM_XOR);
	View_draw_bitmap(&mapview,BitmapSequence_get_bitmap(&bseqTileSet,16),Cursor_get_x(&cursor)*8-Cursor_get_x(&anchor)*8,Cursor_get_y(&cursor)*8-Cursor_get_y(&anchor)*8,BM_NORMAL);
}

void NewLevel()
{
	int width;
	int height;
	bool done;

	width=Map_get_w(&map);
	height=Map_get_h(&map);

	done=FALSE;

	while(!done)
	{
		switch(NewLevelMenu(width,height))
		{
		case 0:
			{
				width=InputBoxNum("Change Width","Input a value for width.\nRange 1-255.",width);
			}break;
		case 1:
			{
				height=InputBoxNum("Change Height","Input a value for height.\nRange 1-255.",height);
			}break;
		case 2:
			{
				Map_dtor(&map,LEAVE_MEMORY);
				Cursor_dtor(&cursor,LEAVE_MEMORY);
				Cursor_dtor(&anchor,LEAVE_MEMORY);
				Map_ctor(&map,width,height);
				Cursor_ctor(&cursor,0,0,Map_get_w(&map),Map_get_h(&map),TRUE,TRUE);
				Cursor_ctor(&anchor,0,0,Map_get_w(&map)-12,Map_get_h(&map)-12,FALSE,FALSE);
				done=TRUE;
			}break;
		case 3:
			{
				done=TRUE;
			}break;
		}
	}
	Map_draw();
	Map_show();
}

int NewLevelMenu(int w,int h)
{
	int mr;
	int i;

	for(i=0;i<5;i++)
	{
		NewMenu[i]=malloc(20);
	}

	sprintf(NewMenu[0],"Width:%d",w);
	sprintf(NewMenu[1],"Height:%d",h);
	sprintf(NewMenu[2],"Create");
	sprintf(NewMenu[3],"Cancel");
	sprintf(NewMenu[4],"");

	mr=SimpleMenuForm("New Level",NewMenu,0,0,160,100,FALSE,main_module.m_process);

	for(i=0;i<5;i++)
	{
		free(NewMenu[i]);
	}

    return(mr);
}

int InputBoxNum(char* title,char* prompt,int num)
{
	char buf[20];
	int mr;
	long val;

	sprintf(buf,"%d",num);

	mr=InputBox(title,prompt,mbOk | mbCancel,19,buf,main_module.m_process);

	switch(mr)
	{
	case mrOk:
		{
			val=atoul(buf);
			if(val<1) return(num);
			if(val>255) return(num);
			return((int)val);
		}break;
	case mrCancel:
		{
			return(num);
		}break;
	}
}

void OpenLevel()
{
	int mr;
	char* buf;
	struct FileInput* pfile;

	buf=malloc(80);

	mr=FileListForm("Open Level...","*.lvl",buf,main_module.m_process);

	if(mr>=0)
	{

		pfile=(struct FileInput*)malloc(sizeof(struct FileInput));

		FileInput_ctor_Ex(pfile,buf);

		Map_dtor(&map,LEAVE_MEMORY);
		Cursor_dtor(&cursor,LEAVE_MEMORY);
		Cursor_dtor(&anchor,LEAVE_MEMORY);
		Map_ctor_Ex(&map,pfile);
		Cursor_ctor(&cursor,0,0,Map_get_w(&map),Map_get_h(&map),TRUE,TRUE);
		Cursor_ctor(&anchor,0,0,Map_get_w(&map)-11,Map_get_h(&map)-11,FALSE,FALSE);

		FileInput_dtor(pfile,FREE_MEMORY);
	}

	free(buf);
}

void SaveLevel()
{
	int mr;
	char* buf;
	struct FileOutput* pfile;

	buf=malloc(80);

	mr=InputBox("Save As...","Input file name.",mbOk | mbCancel,70,buf,main_module.m_process);

	if(mr==mrOk)
	{
		strcat(buf,".lvl");

		pfile=(struct FileOutput*)malloc(sizeof(struct FileOutput));

		FileOutput_ctor_Ex(pfile,buf,TRUE);

		Map_write(&map,pfile);

		FileOutput_dtor(pfile,FREE_MEMORY);
	}

	free(buf);
}

